from tests.e2e.runner.runner import E2ERunner
from tests.fixtures.datasource import FRIENDS_QUESTION
from verse_sdk import observe


class E2ERunnerSync(E2ERunner):
    @observe("e2e.runner.sync", type="trace")
    def run(self):
        response = self._agent.ask(FRIENDS_QUESTION)

        self.setup()
        self.assert_response_len(response)
        self.assert_validity()
        self.teardown()
