from .api import (
    CHARACTER_DETAILS_QUESTION,
    FRIENDS_QUESTION,
    STORY_QUESTION,
    get_all_characters,
    get_all_episodes,
    get_all_items,
    get_all_locations,
    get_all_occupations,
    get_character_by_id,
    get_character_location,
    get_character_occupation,
    get_characters_by_episode,
    get_characters_by_item,
    get_characters_by_location,
    get_characters_by_occupation,
    get_episode_by_id,
    get_episodes_by_character,
    get_episodes_by_item,
    get_episodes_by_location,
    get_favorite_items,
    get_friends,
    get_item_by_id,
    get_items_by_episode,
    get_location_by_episode,
    get_location_by_id,
    get_occupation_by_id,
)

datasource = [
    get_all_characters,
    get_all_episodes,
    get_all_items,
    get_all_locations,
    get_all_occupations,
    get_character_by_id,
    get_character_location,
    get_character_occupation,
    get_characters_by_episode,
    get_characters_by_item,
    get_characters_by_location,
    get_characters_by_occupation,
    get_episode_by_id,
    get_episodes_by_character,
    get_episodes_by_item,
    get_episodes_by_location,
    get_favorite_items,
    get_friends,
    get_item_by_id,
    get_items_by_episode,
    get_location_by_episode,
    get_location_by_id,
    get_occupation_by_id,
]


__all__ = [
    CHARACTER_DETAILS_QUESTION,
    FRIENDS_QUESTION,
    STORY_QUESTION,
    datasource,
]
