LANGFUSE_ENDPOINT = "/api/public/otel/v1/traces"
LANGFUSE_HOST = "LANGFUSE_HOST"
LANGFUSE_PUBLIC_KEY = "LANGFUSE_PUBLIC_KEY"
LANGFUSE_REGION = "LANGFUSE_REGION"
LANGFUSE_REGION_US = "US"
LANGFUSE_SECRET_KEY = "LANGFUSE_SECRET_KEY"
LANGFUSE_URL_EU = f"https://cloud.langfuse.com{LANGFUSE_ENDPOINT}"
LANGFUSE_URL_US = f"https://us.cloud.langfuse.com{LANGFUSE_ENDPOINT}"
