import pytest

from tests.e2e.runner import E2ERunnerAsync


@pytest.mark.e2e
@pytest.mark.parametrize(
    "name",
    [
        "anthropic",
        # No paid account to run in CI/CD
        # "google",
        "langchain",
        "litellm",
        "pydantic_ai",
    ],
)
def test_agent_integration_with_streaming(name: str):
    E2ERunnerAsync(name).run()
